/**
	Definitions for a program
	to deform a very simple mass-spring-dashpot system (only with one!).
	(this use OpenGL primitives and Qt)

	Fraga 8/12/2010
**/
#ifndef SPRING_H
#define SPRING_H

#include <QWidget>
#include <QtOpenGL>
#include <QTimer>
#include <QDoubleSpinBox>

class Spring: public QGLWidget
{
	Q_OBJECT  // must include this if you use Qt signals/slots
public:
	Spring( QWidget *parent=0 );

	void createSpring( void );
	void showSpring( void );
	QSizePolicy sizePolicy() const;

public slots:
	void reset( void ) {
		timer->stop( );
		s.f = 0.0;
    	s.k3 = (s.f * s.dt * s.dt)/s.m;
    	s.x_1 = s.x = s.x1 = 0.0;

		updateGL();
	}
	void apply( void ) { 
		s.k3 = (s.f * s.dt * s.dt)/s.m;
		timer->start( 33 );
	}
	void remove( void ) { 
		s.f = 0.0;
		// s.k3 = (s.f * s.dt * s.dt)/s.m;
		s.k3 = 0.0;
	}
	void setForce( double v ) { 
		s.f = v;
		// fprintf( stderr, "f = %lf\n", s.f );
	}
	void oneStepDeformation( void );

protected:
	void initializeGL( void );
	void resizeGL( int w, int h );
	void paintGL( void );
 
 
private:
	QTimer *timer;

	int windowWidth, windowHeight;   // canvas width and height

	float yrot, xrot;  // Encene rotation

	double angle;

	typedef struct msd {
		double f;
		double k, m, b;
		double dt;  // Initial dt = 0.0333333333;
		double k1, k2, k3;
		
		double x_1, x, x1;
	} MSD;


	MSD s;	
};


#endif // SPRING_H
